import CryptoJS from "./hmac-sha1.js"
import consts from "./consts.js"
import parse from './html-parser.js'

function getSignature() {
	const isIOS = uni.getSystemInfoSync().platform == "ios"
	let nonce = Math.round(Math.random() * 9000000000 + 1000000000).toString(),
		timestamp = Math.round(new Date().getTime() / 1000).toString(),
		sortedParamsStr = [nonce, timestamp, isIOS ? consts.APP_SECRET_IOS : consts.APP_SECRET_ANDRIOD].sort().join(''),
		sign = CryptoJS.SHA1(sortedParamsStr).toString(CryptoJS.enc.Hex);
	return `signature=${sign}&timestamp=${timestamp}&nonce=${nonce}`;
}

function getCommonHeaders(token) {
	const sysInfo = uni.getSystemInfoSync()
	const isIOS = sysInfo.platform == "ios"
	let headers = {
		'AppId': isIOS ? consts.APP_ID_IOS : consts.APP_ID_ANDRIOD,
		'AppVersion': consts.APP_VERSION,
		'Signature': getSignature(),
		'Content-type': 'application/json',
		'X-Request-Id': uuid(),
		'DEVICEID': sysInfo.deviceId
	}
	if (token) {
		headers = Object.assign({}, headers, {
			'Authorization': 'Bearer ' + token
		})
	}
	return headers
};

function request(url, data = {}, header = {}, config = {}, method = 'POST') {
	let _header = {
		'Content-type': 'application/json'
	}
	if (Object.keys(header).length > 0) {
		_header = header
	}
	let showToast = true,
		showLoading = false,
		loadingTitle = '加载中...',
		autoCatch = true
	if (config['showToast'] != undefined && config['showToast'] == false) {
		showToast = false;
	}
	if (config['showLoading'] != undefined && config['showLoading'] == true) {
		showLoading = true;
	}
	if (config['autoCatch'] != undefined && config['autoCatch'] == false) {
		autoCatch = false;
	}
	if (config['loadingTitle']) {
		loadingTitle = config['loadingTitle'];
	}

	return new Promise((resolve, reject) => {
		if (showLoading) {
			uni.showLoading({
				title: loadingTitle,
				mask: true
			})
		}
		uni.request({
			url: url,
			data: data,
			header: _header,
			method: method,
			success: (res) => {
				if (res.statusCode && res.statusCode != 200) {
					if (res.statusCode == 403 && getApp().globalData.globalModalEnable) {
						getApp().globalData.globalModalEnable = false
						uni.showModal({
							title: '温馨提示',
							content: '您的登录信息已失效，请重新登录。',
							showCancel: false,
							confirmText: '立即登录',
							success: () => {
								getApp().globalData.globalModalEnable = true
								uni.reLaunch({
									url: '/pages/temp?target=login',
									success: () => {
										removeStorage(consts
											.STORAGE_USER_TOKEN)
									}
								})
							}
						})
						reject(res)
						return
					} else {
						uni.showToast({
							title: `服务器${res.statusCode}错误`,
							icon: "none"
						})
						console.log(`服务器${res.statusCode}错误，url=${url}`)
						reject(res)
						return
					}
				}
				if (res.data && res.data.code != 0 && autoCatch) {
					if (res.data.code == 8011 && getApp().globalData.globalModalEnable) {
						getApp().globalData.globalModalEnable = false
						uni.showModal({
							title: '温馨提示',
							content: '您的账号已在其他设备登录。如需在本设备使用，请重新登录。',
							showCancel: false,
							confirmText: '立即登录',
							success: () => {
								getApp().globalData.globalModalEnable = true
								uni.reLaunch({
									url: '/pages/temp?target=login',
									success: () => {
										removeStorage(consts
											.STORAGE_USER_TOKEN)
									}
								})
							}
						})
						reject(res.data)
						return
					}
					if (showToast) {
						uni.showToast({
							title: res.data.msg,
							icon: "none"
						})
					}
					reject(res.data)
					if (consts.ENV != 'PROD') {
						console.log(
							`header=${JSON.stringify(_header)}；地址=${url},${method}；入参=${JSON.stringify(data)}；返回=${JSON.stringify(res)}`
						)
					}
					return
				}
				if (consts.ENV != 'PROD') {
					console.log(
						`header=${JSON.stringify(_header)}；地址=${url},${method}；入参=${JSON.stringify(data)}；返回=${JSON.stringify(res.data)}`
					)
				}
				resolve(res.data)
			},
			fail: (err) => {
				if (consts.ENV != 'PROD') {
					console.log(
						`header=${JSON.stringify(_header)}；${url},${method}===>${JSON.stringify(err)}`
					)
				}
				reject(err)
			},
			complete: () => {
				if (showLoading) {
					uni.hideLoading()
				}
			}
		})
	})
}

function setStorage(key, rawData, expire = null) {
	let data = {
			'raw': rawData == undefined ? '' : rawData
		},
		now = new Date().getTime()
	if (expire) {
		data['expireDate'] = now + expire
	}
	uni.setStorageSync(key, data)
}

function getStorage(key) {
	let data = uni.getStorageSync(key),
		now = new Date().getTime()

	if (!data) return ''

	if (!data['expireDate']) return data['raw']

	if (data['expireDate'] < now) return ''

	return data['raw']
}

function removeStorage(key) {
	uni.removeStorageSync(key)
}

function clearStorage() {
	uni.clearStorageSync()
}

/**
 * @description 判断是否为空
 * @param value
 * @returns {boolean}
 */
function isBlank(value) {
	return (
		value == null ||
		false ||
		value === '' ||
		value.trim() === '' ||
		value.toLocaleLowerCase().trim() === 'null'
	)
}

/**
 * @description 校验验证码是否小于6位
 * @param value, cent
 * @returns {boolean}
 */
function isVerify(value, cent) {
	return value.length == cent
}

/**
 * @description 判断是否为数字
 * @param value
 * @returns {boolean}
 */
function isNumber(value) {
	const reg = /^[0-9]*$/
	return reg.test(value)
}

/**
 * @description 判断是否为整数或小数
 * @param value
 * @returns {boolean}
 */
function isDigit(value) {
	const reg = /^[0-9]+(\.?[0-9]+)?$/
	return reg.test(value)
}

/**
 * @description 判断是否是手机号
 * @param value
 * @returns {boolean}
 */
function isPhone(value) {
	const reg = /^1\d{10}$/
	return reg.test(value)
}

/**
 * @description 判断是否是身份证号(第二代)
 * @param value
 * @returns {boolean}
 */
function isIdCard(value) {
	const reg = /^[1-9]\d{5}(18|19|([23]\d))\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\d{3}[0-9Xx]$/
	return reg.test(value)
}

/**
 * @description 判断是否是邀请码
 * @param value
 * @returns {boolean}
 */
function isInvita(value) {
	const reg = /^[A-Za-z0-9]{6}$/
	return reg.test(value)
}

/**
 * @description 判断是不是json字符串
 * @param value
 * @returns {boolean}
 */
function isJsonString(value) {
	try {
		if (typeof JSON.parse(value) == "object") return true
	} catch (e) {}
	return false
}

/**
 * @description 补0
 * @param value
 * @returns {value}
 */
function addZero(value) {
	let temp = value + '';
	if (temp.length === 0) {
		return '00'
	} else if (temp.length === 1) {
		return '0' + temp
	}
	return value
}

/**
 * @description 格式化时间显示
 * @param value format
 * @returns {value}
 */
function formatDate(date, format) {
	let year = addZero(date.getFullYear()),
		month = addZero(date.getMonth() + 1),
		day = addZero(date.getDate()),
		hours = addZero(date.getHours()),
		minutes = addZero(date.getMinutes()),
		seconds = addZero(date.getSeconds())
	switch (format) {
		case 'y-m-d':
			return `${year}-${month}-${day}`
			break;
		case 'y-m-d h:m':
			return `${year}-${month}-${day} ${hours}:${minutes}`
			break;
		case 'y-m-d h:m:s':
			return `${year}-${month}-${day} ${hours}:${minutes}:${seconds}`
			break;

		case 'm.d':
			return `${month}.${day}`
			break;
		case 'y.m.d':
			return `${year}.${month}.${day}`
			break;
		case 'y.m.d h:m':
			return `${year}.${month}.${day} ${hours}:${minutes}`
			break;
		case 'y.m.d h:m:s':
			return `${year}.${month}.${day} ${hours}:${minutes}:${seconds}`
			break;

		case 'h:m':
			return `${hours}:${minutes}`
			break;
		case 'h:m:s':
			return `${hours}:${minutes}:${seconds}`
			break;

		case 'y/m/d':
			return `${year}/${month}/${day}`
			break;
		case 'y/m/d h:m':
			return `${year}/${month}/${day} ${hours}:${minutes}`
			break;
		default:
			return `${year}/${month}/${day} ${hours}:${minutes}:${seconds}`
			break;
	}
}

// 获取指定时间 多少天之内的全部日期
function getDateMiusDays(date, days) {
	let time = date.getTime()
	let dates = []
	for (let i = 0; i <= days; i++) {
		let before30Time = time - (60 * 60 * 24 * 1000 * i)
		let newDate = new Date(before30Time)
		let year = addZero(newDate.getFullYear())
		let month = addZero(newDate.getMonth() + 1)
		let day = addZero(newDate.getDate())
		dates.push(`${year}-${month}-${day}`)
	}
	return dates.reverse()
}

/**
 * 获取2个日期之间的天
 * @param {Object} srart 开始时间
 * @param {Object} end 结束时间
 */
function getDateBetweenDays(srart, end) {

	let bet = end.getTime() - srart.getTime()
	let dayDate = 60 * 60 * 24 * 1000
	let betDays = Math.ceil(bet / dayDate)
	// console.log(srart, end, bet, dayDate, betDays, 'betDays====>>>')
	return getDateMiusDays(end, betDays)
}

/**
 * @description 格式化数字显示
 * @param value decimals dec_point thousands_sep roundtag
 * @returns {value}
 */
function formatNumber(value, decimals = '8', dec_point = '.', thousands_sep = ',', roundtag = 'floor') {
	/*
	 * 参数说明：
	 * value：要格式化的数字
	 * decimals：保留几位小数
	 * dec_point：小数点符号
	 * thousands_sep：千分位符号
	 * roundtag:舍入参数，默认 "ceil" 向上取,"floor"向下取,"round" 四舍五入
	 * */
	if (value == undefined) return
	if (roundtag != 'round' && Math.abs(value) != value) {
		if (roundtag == 'ceil') {
			roundtag = 'floor'
		} else if (roundtag == 'floor') {
			roundtag = 'ceil'
		}
	}
	value = (value + '').replace(/[^0-9+-Ee.]/g, '')
	let n = !isFinite(+value) ? 0 : +value,
		prec = !isFinite(+decimals) ? 0 : Math.abs(decimals),
		sep = (typeof thousands_sep === 'undefined') ? ',' : thousands_sep,
		dec = (typeof dec_point === 'undefined') ? '.' : dec_point,
		s = '',
		toFixedFix = function(n, prec) {
			let k = Math.pow(10, prec)
			return '' + parseFloat(Math[roundtag](parseFloat((n * k).toFixed(prec * 2))).toFixed(prec * 2)) / k
		}
	s = (prec ? toFixedFix(n, prec) : '' + Math.round(n)).split('.')
	let re = /(-?\d+)(\d{3})/
	while (re.test(s[0])) {
		s[0] = s[0].replace(re, "$1" + sep + "$2")
	}
	if ((s[1] || '').length < prec) {
		s[1] = s[1] || '';
		s[1] += new Array(prec - s[1].length + 1).join('0')
	}
	return s.join(dec)
}

/**
 * @description 格式化html
 * @param value decimals dec_point thousands_sep roundtag
 * @returns {value}
 */
function formatHTML(value) {
	return parse.parseHtml(value)
}

/**
 * @description 格式化手机号
 * @param value 
 * @returns {value}
 */
function formatMobile(value) {
	let mobile = String(value)
	if (mobile.length > 7) {
		let rep = mobile.length - 7;
		let str1 = mobile.slice(0, 3);
		let str2 = mobile.slice(-4);
		return str1 + '*'.repeat(rep) + str2
	}
	return mobile
}

/**
 * @description px to rpx
 * @param value
 * @returns {value}
 */
function px2upx(value) {
	return Math.round(value / (uni.upx2px(100) / 100))
}

/**
 * @description 生成uuid
 * @param value
 * @returns {value}
 */
function uuid(len = 32) {
	const num = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890'
	let str = ''
	for (let i = 0; i < len; i++) {
		str += num.charAt(Math.floor(Math.random() * num.length))
	}
	return str
}

function readPath(path) {
	if (path.indexOf('http') > -1) {
		return Promise.resolve(path)
	}
	return new Promise((resolve, reject) => {
		plus.io.resolveLocalFileSystemURL(
			`_www/static/models/`,
			(entry) => {
				const reader = entry.createReader()
				reader.readEntries(
					(entries) => {
						entries.map(async item => {
							if (item.name == path) {
								let _url = await item.__remoteURL__
								resolve(_url)
							}
						})
					},


					(err) => {
						reject(err)
					}
				)
			},
			(err) => {
				reject(err)
			}
		)
	})
}

module.exports = {
	getCommonHeaders,
	request,

	setStorage,
	getStorage,
	removeStorage,
	clearStorage,

	isBlank,
	isVerify,
	isNumber,
	isDigit,
	isPhone,
	isIdCard,
	isInvita,
	isJsonString,

	addZero,
	formatDate,
	getDateMiusDays,
	getDateBetweenDays,
	formatNumber,
	formatHTML,
	formatMobile,

	px2upx,
	uuid,

	readPath
}
